@@ -120,7 +120,7 @@ def _get_nvrtc_version():
 
 
 # Known archs for Tegra/Jetson/Xavier/etc
-_tegra_archs = ('53', '62', '72')
+_tegra_archs = ('32', '53', '62', '72')
 
 
 @_util.memoize()
@@ -132,9 +132,13 @@ def _get_max_compute_capability():
     elif major == 11 and minor == 0:
         # CUDA 11.0
         nvrtc_max_compute_capability = '80'
-    else:
-        # CUDA 11.1 / 11.2 / 11.3 / 11.4
+    elif major == 11 and minor < 4:
+        # CUDA 11.1 / 11.2 / 11.3
         nvrtc_max_compute_capability = '86'
+    else:
+        # CUDA 11.4+
+        nvrtc_max_compute_capability = '87'
+
     return nvrtc_max_compute_capability
 
 
@@ -691,6 +695,8 @@ class _NVRTCProgram(object):
                 return nvrtc.getCUBIN(self.ptr), mapping
             elif self.method == 'ptx':
                 return nvrtc.getPTX(self.ptr), mapping
+            # TODO(leofang): support JIT LTO using nvrtc.getNVVM()?
+            # need -dlto and -arch=compute_XX
             else:
                 raise RuntimeError('Unknown NVRTC compile method')
         except nvrtc.NVRTCError:
@@ -760,12 +766,20 @@ def _preprocess_hipcc(source, options):
 
 def _preprocess_hiprtc(source, options):
     # source is ignored
-    prog = _NVRTCProgram(
+    if _cuda_hip_version >= 40400000:
+        # HIP runtime headers can be no longer explicitly included on ROCm 4.5+
+        code = '''
+        // hiprtc segfaults if the input code is empty
+        __global__ void _cupy_preprocess_dummy_kernel_() { }
         '''
+    else:
+        code = '''
         // hiprtc segfaults if the input code is empty
         #include <hip/hip_runtime.h>
         __global__ void _cupy_preprocess_dummy_kernel_() { }
-        ''')
+        '''
+
+    prog = _NVRTCProgram(code)
     try:
         result, _ = prog.compile(options)
     except CompileException as e:
@@ -782,7 +796,12 @@ _hip_extra_source = None
 
 
 def _convert_to_hip_source(source, extra_source, is_hiprtc):
-    if (not is_hiprtc) or (is_hiprtc and _cuda_hip_version >= 402):
+    if not is_hiprtc:
+        return '#include <hip/hip_runtime.h>\n' + source
+    if _cuda_hip_version >= 40400000:
+        # HIP runtime headers can be no longer explicitly included on ROCm 4.5+
+        return source
+    if _cuda_hip_version >= 402:
         # "-I" is fixed on ROCm 4.2.0+
         return '#include <hip/hip_runtime.h>\n' + source
 
